@@ -48,8 +48,8 @@ DEFAULT_PORTS = {
 # 'mailto:user@google.com' instead of 'mailto://user@google.com'. Scheme
 # strings are lowercase.
 #
-# TODO(grun): Support schemes separated by just ':', not '://' without having an
-# explicit list. There are many such schemes in various URIs.
+# TODO(grun): Support schemes separated by just ':', not '://' without having
+# an explicit list. There are many such schemes in various URIs.
 COLON_SEPARATED_SCHEMES = [
     'mailto',
     ]
@@ -169,7 +169,7 @@ class Path(object):
         Returns: <self>.
         """
         if str(self):
-            normalized = normpath(str(self)) + '/'*self.isdir
+            normalized = normpath(str(self)) + ('/' * self.isdir)
             if normalized.startswith('//'):  # http://bugs.python.org/636648
                 normalized = '/' + normalized.lstrip('/')
             self.load(normalized)
@@ -222,7 +222,7 @@ class Path(object):
                 segments = ['', '']
             else:
                 segments.insert(0, '')
-        return self._path_from_segments(segments, quoted=True)
+        return self._path_from_segments(segments)
 
     def __repr__(self):
         return "%s('%s')" % (self.__class__.__name__, str(self))
@@ -240,27 +240,21 @@ class Path(object):
                 if not is_valid_encoded_path_segment(segment):
                     s = ("Improperly encoded path string received: '%s'. "
                          "Proceeding, but did you mean '%s'?" %
-                         (path, self._path_from_segments(segments, True)))
+                         (path, self._path_from_segments(segments)))
                     warnings.warn(s, UserWarning)
                     break
         return map(urllib.unquote, segments)
 
-    def _path_from_segments(self, segments, quoted=True):
+    def _path_from_segments(self, segments):
         """
-        Combine the provided path segments <segments> into a path
-        string. If <quoted> is True, each path segment will be
-        quoted. If <quoted> is False, each path segment will be
-        unquoted.
+        Combine the provided path segments <segments> into a path string. Path
+        segments in <segments> will be quoted.
 
-        Returns: A path string, with either quoted or unquoted path
-        segments.
+        Returns: A path string with quoted path segments.
         """
-        segments_str = ''.join(segments)
-        if quoted and '%' not in segments_str:
-            segments = map(lambda s: urllib.quote(s, self.SAFE_SEGMENT_CHARS),
-                           segments)
-        elif not quoted and '%' in segments_str:
-            segments = map(urllib.unquote, segments)
+        if '%' not in ''.join(segments):  # Don't double-encode the path.
+            segments = [urllib.quote(u2utf8(segment), self.SAFE_SEGMENT_CHARS)
+                        for segment in segments]
         return '/'.join(segments)
 
 
@@ -511,6 +505,7 @@ class Query(object):
         """
         pairs = []
         for key, value in self.params.iterallitems():
+            key, value = u2utf8(key), u2utf8(value)
             quoted_key = urllib.quote_plus(str(key), self.SAFE_KEY_CHARS)
             quoted_value = urllib.quote_plus(str(value), self.SAFE_VALUE_CHARS)
             pair = '='.join([quoted_key, quoted_value])
@@ -584,8 +579,9 @@ class Query(object):
             pairs = map(lambda p: (p[0], '') if len(p) == 1
                         else (p[0], p[1]), pairs)
             for key, value in pairs:
-                if (not is_valid_encoded_query_key(key) or
-                    not is_valid_encoded_query_value(value)):
+                valid_key = is_valid_encoded_query_key(key)
+                valid_value = is_valid_encoded_query_value(value)
+                if not valid_key or not valid_value:
                     s = ("Improperly encoded query string received: '%s'. "
                          "Proceeding, but did you mean '%s'?" %
                          (querystr, urllib.urlencode(pairs)))
@@ -595,8 +591,9 @@ class Query(object):
         items = []
         parsed_items = urlparse.parse_qsl(querystr, keep_blank_values=True)
         for (key, value), pairstr in izip(parsed_items, pairstrs):
-            # Empty value without '=', like '?sup'.
-            if key == urllib.quote_plus(pairstr):
+            # Empty value without '=', like '?sup'. Encode to utf8 to handle
+            # unicode strings.
+            if key.encode('utf8') == urllib.quote_plus(pairstr.encode('utf8')):
                 value = None
             items.append((key, value))
         return items
@@ -796,7 +793,7 @@ class furl(URLPathCompositionInterface, QueryCompositionInterface,
       scheme://username:password@host:port/path?query#fragment
 
     Attributes:
-      DEFAULT_PORTS: 
+      DEFAULT_PORTS:
       strict: Boolean whether or not UserWarnings should be raised if
         improperly encoded path, query, or fragment strings are provided
         to methods that take such strings, like load(), add(), set(),
@@ -839,9 +836,7 @@ class furl(URLPathCompositionInterface, QueryCompositionInterface,
         self._port = None
         self._scheme = None
 
-        if isinstance(url, unicode):
-            url = url.encode('utf-8')
-        url = str(url)
+        url = str(u2utf8(url))
 
         # urlsplit() raises a ValueError on malformed IPv6 addresses in
         # Python 2.7+. In Python <= 2.6, urlsplit() doesn't raise a
@@ -1074,8 +1069,8 @@ class furl(URLPathCompositionInterface, QueryCompositionInterface,
             <fragment_args>, and/or <fragment_separator>) are provided.
         Returns: <self>.
         """
-        if (netloc is not _absent and
-            (host is not _absent or port is not _absent)):
+        netloc_present = netloc is not _absent
+        if (netloc_present and (host is not _absent or port is not _absent)):
             s = ('Possible parameter overlap: <netloc> and <host> and/or '
                  '<port> provided. See furl.set() documentation for more '
                  'details.')
@@ -1229,7 +1224,6 @@ class furl(URLPathCompositionInterface, QueryCompositionInterface,
 
 
 def _get_scheme(url):
-    scheme = None
     if url.lstrip().startswith('//'):  # Protocol relative URL.
         return ''
     beforeColon = url[:max(0, url.find(':'))]
@@ -1241,7 +1235,6 @@ def _get_scheme(url):
 def _set_scheme(url, newscheme):
     scheme = _get_scheme(url)
     newscheme = newscheme or ''
-    separator = ':' if scheme in COLON_SEPARATED_SCHEMES else '://'
     newseparator = ':' if newscheme in COLON_SEPARATED_SCHEMES else '://'
     if scheme == '':  # Protocol relative URL.
         url = '%s:%s' % (newscheme, url)
@@ -1390,11 +1383,18 @@ def is_valid_port(port):
 def callable_attr(obj, attr):
     return hasattr(obj, attr) and callable(getattr(obj, attr))
 
+
+def u2utf8(s):
+    if isinstance(s, unicode):
+        return s.encode('utf8')
+    return s
+
+
 #
-# TODO(grun): These functions need to be expanded to reflect the fact
-# that the valid encoding for a URL Path segment is different from a
-# Fragment Path segment, and valid URL Query key and value encoding is
-# different than valid Fragment Query key and value encoding.
+# TODO(grun): These regex functions need to be expanded to reflect the
+# fact that the valid encoding for a URL Path segment is different from
+# a Fragment Path segment, and valid URL Query key and value encoding
+# is different than valid Fragment Query key and value encoding.
 #
 # For example, '?' and '#' don't need to be encoded in Fragment Path
 # segments but they must be encoded in URL Path segments.
@@ -1432,11 +1432,13 @@ VALID_ENCODED_PATH_SEGMENT_REGEX = re.compile(
 def is_valid_encoded_path_segment(segment):
     return bool(VALID_ENCODED_PATH_SEGMENT_REGEX.match(segment))
 
+
 VALID_ENCODED_QUERY_KEY_REGEX = re.compile(
     r'^([\w\-\.\~\:\@\!\$\&\'\(\)\*\+\,\;\/\?]|(\%[\da-fA-F][\da-fA-F]))*$')
 def is_valid_encoded_query_key(key):
     return bool(VALID_ENCODED_QUERY_KEY_REGEX.match(key))
 
+
 VALID_ENCODED_QUERY_VALUE_REGEX = re.compile(
     r'^([\w\-\.\~\:\@\!\$\&\'\(\)\*\+\,\;\/\?\=]|(\%[\da-fA-F][\da-fA-F]))*$')
 def is_valid_encoded_query_value(value):
