@@ -483,6 +483,12 @@ class TestPath(unittest.TestCase):
         p = furl.Path('/asdf')
         assert p
 
+    def test_unicode(self):
+        path = u'/wiki/Восход'
+        path_encoded = '/wiki/%D0%92%D0%BE%D1%81%D1%85%D0%BE%D0%B4'
+        p = furl.Path(path)
+        assert str(p) == path_encoded
+
 
 class TestQuery(unittest.TestCase):
 
@@ -622,22 +628,22 @@ class TestQuery(unittest.TestCase):
             assert len(q.params) == 0
 
         # List of keys to remove.
-        q = furl.Query([('a','1'), ('b', '2'), ('b', '3'), ('a', '4')])
+        q = furl.Query([('a', '1'), ('b', '2'), ('b', '3'), ('a', '4')])
         q.remove(['a', 'b'])
         assert not q.params.items()
 
         # List of items to remove.
-        q = furl.Query([('a','1'), ('b', '2'), ('b', '3'), ('a', '4')])
+        q = furl.Query([('a', '1'), ('b', '2'), ('b', '3'), ('a', '4')])
         q.remove([('a', '1'), ('b', '3')])
         assert q.params.allitems() == [('b', '2'), ('a', '4')]
 
         # Dictionary of items to remove.
-        q = furl.Query([('a','1'), ('b', '2'), ('b', '3'), ('a', '4')])
-        q.remove({'b':'3', 'a':'1'})
+        q = furl.Query([('a', '1'), ('b', '2'), ('b', '3'), ('a', '4')])
+        q.remove({'b': '3', 'a': '1'})
         assert q.params.allitems() == [('b', '2'), ('a', '4')]
 
         # Multivalue dictionary of items to remove.
-        q = furl.Query([('a','1'), ('b', '2'), ('b', '3'), ('a', '4')])
+        q = furl.Query([('a', '1'), ('b', '2'), ('b', '3'), ('a', '4')])
         omd = omdict1D([('a', '4'), ('b', '3'), ('b', '2')])
         q.remove(omd)
         assert q.params.allitems() == [('a', '1')]
@@ -710,6 +716,20 @@ class TestQuery(unittest.TestCase):
         q.params['slrp'] = ''
         assert str(q) == 'prp=&slrp=' and q.params['slrp'] == ''
 
+    def test_unicode(self):
+        key, value = u'Восход_', u'testä'
+        key_encoded = urllib.quote_plus(key.encode('utf8'))
+        value_encoded = urllib.quote_plus(value.encode('utf8'))
+
+        q = furl.Query(u'%s=%s' % (key, value))
+        assert q.params[key] == value
+        assert str(q) == '%s=%s' % (key_encoded, value_encoded)
+
+        q = furl.Query()
+        q.params[key] = value
+        assert q.params[key] == value
+        assert str(q) == '%s=%s' % (key_encoded, value_encoded)
+
     def _quote_items(self, items):
         # Calculate the expected querystring with proper query encoding.
         #   Valid query key characters: "/?:@-._~!$'()*,;"
@@ -940,29 +960,46 @@ class TestFurl(unittest.TestCase):
         items = urlparse.parse_qsl(urlparse.urlsplit(url).query, True)
         return (key, val) in items
 
-
     def test_unicode(self):
-        url = u'http://ru.wikipedia.org/wiki/Восход_(ракета-носитель)'
-        f = furl.furl(url)  # Accept a unicode URL without raising an exception.
+        path = u'Восход'
+        path_encoded = '%D0%92%D0%BE%D1%81%D1%85%D0%BE%D0%B4'
+
+        key, value = u'testö', u'testä'
+        key_encoded = urllib.quote_plus(key.encode('utf8'))
+        value_encoded = urllib.quote_plus(value.encode('utf8'))
+
+        base_url = u'http://pumps.ru/'
+        url = base_url + path
+        url_encoded = base_url + path_encoded
+
+        f = furl.furl(url)  # Accept unicode without raising an exception.
+        assert f.url == url_encoded
+
+        f = furl.furl(base_url)
+        f.args[key] = value
+        assert f.args[key] == value  # Unicode keys and values aren't modified.
         assert not isinstance(f.url, unicode)  # URLs cannot contain unicode.
+        f.path.segments = [path]
+        assert f.path.segments == [path]  # Unicode segments aren't modified.
+        assert f.url == url_encoded + '?%s=%s' % (key_encoded, value_encoded)
 
     def test_scheme(self):
         assert furl.furl().scheme is None
         assert furl.furl('').scheme is None
-        
+
         # Lowercase.
         assert furl.furl('/sup/').set(scheme='PrOtO').scheme == 'proto'
-        
+
         # No scheme.
         for url in ['sup.txt', '/d/sup', '#flarg']:
             f = furl.furl(url)
             assert f.scheme is None and f.url == url
-        
+
         # Protocol relative URLs.
         for url in ['//', '//sup.txt', '//arc.io/d/sup']:
             f = furl.furl(url)
             assert f.scheme == '' and f.url == url
-        
+
         f = furl.furl('//sup.txt')
         assert f.scheme == ''
         f.scheme = None
